@@ -921,6 +921,7 @@ LIBTCCAPI TCCState *tcc_new(const char *init_lib_path)
     s->output_type = TCC_OUTPUT_MEMORY;
     preprocess_new();
     s->include_stack_ptr = s->include_stack;
+    s->static_link = 0;
 
     /* we add dummy defines for some special macros to speed up tests
        and to have working defined() */
@@ -999,6 +1000,9 @@ LIBTCCAPI TCCState *tcc_new(const char *init_lib_path)
 #else
     tcc_define_symbol(s, "__WCHAR_TYPE__", "int");
 #endif
+#ifdef __LP64__
+    tcc_define_symbol(s, "__LP64__", "1");
+#endif
 
 #ifndef TCC_TARGET_PE
     /* glibc defines */
@@ -1210,6 +1214,18 @@ ST_FUNC int tcc_add_file_internal(TCCState *s1, const char *filename, int flags)
 #endif
         tcc_error_noabort("unrecognized ELF file");
         goto the_end;
+    } else {
+#ifdef TCC_IS_NATIVE
+        if (s1->output_type == TCC_OUTPUT_MEMORY) {
+            /* If it's not an ELF file, but dlopen() can open it, do that. */
+            void *h;
+            h = dlopen(filename, RTLD_GLOBAL | RTLD_LAZY);
+            if (h) {
+                ret = 0;
+                goto the_end;
+            }
+        }
+#endif
     }
 
     if (memcmp((char *)&ehdr, ARMAG, 8) == 0) {
@@ -1290,7 +1306,15 @@ LIBTCCAPI int tcc_add_library(TCCState *s, const char *libraryname)
     const char *libs[] = { "%s/%s.def", "%s/lib%s.def", "%s/%s.dll", "%s/lib%s.dll", "%s/lib%s.a", NULL };
     const char **pp = s->static_link ? libs + 4 : libs;
 #else
+#  ifdef SHOBJEXT
+#    define str1(s) str2(s)
+#    define str2(s) #s
+    const char *libs[] = { "%s/lib%s." str1(SHOBJEXT), "%s/lib%s.a", NULL };
+#    undef str1
+#    undef str2
+#  else
     const char *libs[] = { "%s/lib%s.so", "%s/lib%s.a", NULL };
+#  endif
     const char **pp = s->static_link ? libs + 1 : libs;
 #endif
     while (*pp) {
