@@ -5,15 +5,13 @@ namespace eval tcc4tcl {
 	variable count
 
 	set dir [file dirname [info script]]
+
 	if {[info command ::tcc4tcl] == ""} {
 		catch { load {} tcc4tcl }
 	}
 	if {[info command ::tcc4tcl] == ""} {
 		load [file join $dir tcc4tcl[info sharedlibextension]] tcc4tcl
 	}
-	if {[info command ::tcc4tcl] == ""} {
-		error "Unable to load tcc4tcl shared library"
-	}
 
 	set count 0
 
@@ -22,7 +20,6 @@ namespace eval tcc4tcl {
 		variable count
 
 		set handle ::tcc4tcl::tcc_[incr count]
-		set tcc_handle ::tcc4tcl::tcc_[incr count]
 
 		if {$output == ""} {
 			set type "memory"
@@ -34,7 +31,7 @@ namespace eval tcc4tcl {
 			}
 		}
 
-		array set $handle [list tcc $tcc_handle code "" type $type filename $output package $pkgName]
+		array set $handle [list code "" type $type filename $output package $pkgName add_inc_path "" add_lib_path "" add_lib ""]
 
 		proc $handle {cmd args} [string map [list @@HANDLE@@ $handle] {
 			set handle {@@HANDLE@@}
@@ -60,16 +57,49 @@ namespace eval tcc4tcl {
 		return $handle
 	}
 
-	proc _linktclcommand {cSymbol tclCommand} {
+	proc _linktclcommand {handle cSymbol tclCommand} {
 		upvar #0 $handle state
 
 		lappend state(procs) $cSymbol $tclCommand
 	}
 
+	proc _add_include_path {handle args} {
+		upvar #0 $handle state
+
+		lappend state(add_inc_path) {*}$args
+	}
+
+	proc _add_library_path {handle args} {
+		upvar #0 $handle state
+
+		lappend state(add_lib_path) {*}$args
+	}
+
+	proc _add_library {handle args} {
+		upvar #0 $handle state
+
+		lappend state(add_lib) {*}$args
+	}
+
+	proc _cwrap {handle name adefs rtype} {
+		upvar #0 $handle state
+
+		set wrap [uplevel 1 [list ::tcc4tcl::wrap $name $adefs $rtype "#" "" 1]]
+
+		set wrapped [lindex $wrap 0]
+		set wrapper [lindex $wrap 1]
+		set tclname [lindex $wrap 2]
+
+		append state(code) $wrapped "\n"
+		append state(code) $wrapper "\n"
+
+		lappend state(procs) $name $tclname
+	}
+
 	proc _cproc {handle name adefs rtype {body "#"}} {
 		upvar #0 $handle state
 
-		set wrap [::tcc4tcl::wrap $name $adefs $rtype $body]
+		set wrap [uplevel 1 [list ::tcc4tcl::wrap $name $adefs $rtype $body]]
 
 		set wrapped [lindex $wrap 0]
 		set wrapper [lindex $wrap 1]
@@ -93,6 +123,11 @@ namespace eval tcc4tcl {
 		set state(tk) 1
 	}
 
+	proc _delete {handle} {
+		rename $handle ""
+		unset $handle
+	}
+
 	proc _go {handle {outputOnly 0}} {
 		variable dir
 
@@ -100,6 +135,12 @@ namespace eval tcc4tcl {
 
 		set code $state(code)
 
+		if {$state(type) == "exe" || $state(type) == "dll"} {
+			if {[info exists state(procs)] && [llength $state(procs)] > 0} {
+				set code "int _initProcs(Tcl_Interp *interp);\n\n$code"
+			}
+		}
+
 		if {[info exists state(tk)]} {
 			set code "#include <tk.h>\n$code"
 		}
@@ -168,7 +209,23 @@ namespace eval tcc4tcl {
 			}
 		}
 
-		tcc4tcl $dir $tcc_type tcc
+		if {[info command ::tcc4tcl] == ""} {
+			return -code error "Unable to load tcc4tcl library"
+		}
+
+		::tcc4tcl $dir $tcc_type tcc
+
+		foreach path $state(add_inc_path) {
+			tcc add_include_path $path
+		}
+
+		foreach path $state(add_lib_path) {
+			tcc add_library_path $path
+		}
+
+		foreach lib $state(add_lib) {
+			tcc add_library $lib
+		}
 
 		switch -- $state(type) {
 			"memory" {
@@ -230,7 +287,7 @@ proc ::tcc4tcl::cproc {name adefs rtype {body "#"}} {
 	return [$handle go]
 }
 
-proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""}} {
+proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""} {includePrototype 0}} {
 	if {$cname == ""} {
 		set cname c_[tcc4tcl::cleanname $name]
 	}
@@ -285,12 +342,22 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""}} {
 	}
 
 	# Create wrapped function
+	if {[llength $cargs] != 0} {
+		set cargs_str [join $cargs {, }]
+	} else {
+		set cargs_str "void"
+	}
+
 	if {$body ne "#"} {
-		append code "static $rtype2 ${cname}([join $cargs {, }]) \{\n"
+		append code "static $rtype2 ${cname}($cargs_str) \{\n"
 		append code $body
 		append code "\}\n"
 	} else {
-		append code "#define $cname $name" "\n"
+		set cname [namespace tail $name]
+
+		if {$includePrototype} {
+			append code "$rtype2 ${cname}($cargs_str);\n"
+		}
 	}
 
 	# Create wrapper function
@@ -303,11 +370,12 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""}} {
 	##   char*
 	##   Tcl_Obj*
 	##   void*
+	##   Tcl_WideInt
 	foreach x $varnames {
 		set t $types($x)
 
 		switch -- $t {
-			int - long - float - double - char* - Tcl_Obj* {
+			int - long - float - double - char* - Tcl_WideInt - Tcl_Obj* {
 				append cbody "  $types($x) _$x;" "\n"
 			}
 			default {
@@ -337,6 +405,10 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""}} {
 				append cbody "  if (Tcl_GetLongFromObj(ip, objv\[$n], &_$x) != TCL_OK)"
 				append cbody "    return TCL_ERROR;" "\n"
 			}
+			Tcl_WideInt {
+				append cbody "  if (Tcl_GetWideIntFromObj(ip, objv\[$n], &_$x) != TCL_OK)"
+				append cbody "    return TCL_ERROR;" "\n"
+			}
 			float {
 				append cbody "  {" "\n"
 				append cbody "    double t;" "\n"
@@ -377,9 +449,9 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""}} {
 	#   dstring   (TCL_DYNAMIC char*)
 	#   vstring   (TCL_VOLATILE char*)
 	#   default   (Tcl_Obj*)
-	#   wide
+	#   Tcl_WideInt
 	switch -- $rtype {
-		void - ok - int - long - float - double - wide {}
+		void - ok - int - long - float - double - Tcl_WideInt {}
 		default {
 			append cbody "  if (rv == NULL) {\n"
 			append cbody "    return(TCL_ERROR);\n"
@@ -392,6 +464,7 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""}} {
 		ok             { append cbody "  return rv;" "\n" }
 		int            { append cbody "  Tcl_SetIntObj(Tcl_GetObjResult(ip), rv);" "\n" }
 		long           { append cbody "  Tcl_SetLongObj(Tcl_GetObjResult(ip), rv);" "\n" }
+		Tcl_WideInt    { append cbody "  Tcl_SetWideIntObj(Tcl_GetObjResult(ip), rv);" "\n" }
 		float          -
 		double         { append cbody "  Tcl_SetDoubleObj(Tcl_GetObjResult(ip), rv);" "\n" }
 		char*          { append cbody "  Tcl_SetResult(ip, rv, TCL_STATIC);" "\n" }
@@ -412,4 +485,4 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""}} {
 
 namespace eval tcc4tcl {namespace export cproc new}
 
-package provide tcc4tcl "0.20"
+package provide tcc4tcl "0.21"
